function [U,V,E,mu,rho,p] = 0.5gLPCA(X,W,beta,k,mu,rho)
%% Written by Chun-Mei Feng in Qufu Normal University 2017/2/4
 
% input: vector data X (size:m*n,m:orignal data dim,n:data num); 
%        graph data W(size:n*n)
%        beta:weighting parameter
%        k:reduced dimension
%        C:Lagrangian multipliers
%        
% output:embedding vector V (size:n*k,k:reduction dim);
%        projection U (size:m*k)
%        auxiliary variable E (size: m*n)
% obj: J = \|X-UV^T\|^0.5+alf*trV^T*(D-W)*V    s.t. V^T*V=I
% note that the parameter beta is the replacement of balanced parameter alf, see Eq.(21) in our paper


% Laplacian
d = sum(W);
D = diag(d);
L = D - W;
[m,n]=size(X);
% initialize
C = zeros( m, n);
E = zeros( m, n);
I = eye(n,n);

for ii=1:maxIter
X=X-E-C/mu;
K = X'*X;
sk = max(eig(K));
sl = max(eig(L));
e = ones(n,1);
KL = (1 - beta)*(I - K/sk) + 2*beta/mu*(L/sl + (e*e')/n);   
%============================ Compute the optimal V
[U0,V0] = eig(KL);
val = diag(V0);
[~,ind] = sort(val);
V = U0(:,ind(1:k));

% =========================== Compute the optimal U
U = X*V;

%============================ Compute the optimal E
temp = X-U*V'-C/mu;
    [U2,sigma,V2] = svd(temp,'econ');
    sigma = diag(sigma);  
    lamda = 2/mu;
    thred = ((54.^(1/3))/4)*(lamda.^(2/3));
    svp = length(find(sigma>thred));
    if svp>=1
        Fai_lamda_t = acos((lamda/8)*((sigma(1:svp)/3).^(-3/2)));
        sigma = ((2/3)*sigma(1:svp)).*(1+cos(((2/3)*pi)-((2/3)*Fai_lamda_t)));
    else
        svp = 1;
        sigma = 0;
    end
    E = U2(:,1:svp)*diag(sigma)*V2(:,1:svp)';
    
%============================== Compute the optimal C
C=C+mu*(E-X+U*V');

%============================== Compute the optimal mu
mu=rho*mu;
end
